@@ -551,6 +551,19 @@ impl StreamListener {
         let (socket, sa) = ready!(self.socket.poll_accept_priv(cx))?;
         Poll::Ready(Ok((Stream::from_socket(socket)?, sa)))
     }
+
+    /// Constructs a new [StreamListener] from the given raw file descriptor.
+    ///
+    /// The file descriptor must have been set to non-blocking mode.
+    ///
+    /// This function *consumes ownership* of the specified file descriptor.
+    /// The returned object will take responsibility for closing it when the object goes out of scope.
+    ///
+    /// # Safety
+    /// If the passed file descriptor is invalid, undefined behavior may occur.
+    pub unsafe fn from_raw_fd(fd: RawFd) -> Result<Self> {
+        Ok(Self { socket: Socket::from_raw_fd(fd)? })
+    }
 }
 
 impl AsRef<Socket<Stream>> for StreamListener {
@@ -565,6 +578,20 @@ impl AsRawFd for StreamListener {
     }
 }
 
+impl FromRawFd for StreamListener {
+    /// Constructs a new instance of `Self` from the given raw file
+    /// descriptor.
+    ///
+    /// The file descriptor must have been set to non-blocking mode.
+    ///
+    /// # Panics
+    /// Panics when the conversion fails.
+    /// Use [StreamListener::from_raw_fd] for a non-panicking variant.
+    unsafe fn from_raw_fd(fd: RawFd) -> Self {
+        Self::from_raw_fd(fd).expect("from_raw_fd failed")
+    }
+}
+
 /// An L2CAP stream between a local and remote socket (sequenced, reliable, two-way, connection-based).
 #[derive(Debug)]
 pub struct Stream {
@@ -656,6 +683,19 @@ impl Stream {
 
         self.socket.poll_send_priv(cx, buf)
     }
+
+    /// Constructs a new [Stream] from the given raw file descriptor.
+    ///
+    /// The file descriptor must have been set to non-blocking mode.
+    ///
+    /// This function *consumes ownership* of the specified file descriptor.
+    /// The returned object will take responsibility for closing it when the object goes out of scope.
+    ///
+    /// # Safety
+    /// If the passed file descriptor is invalid, undefined behavior may occur.
+    pub unsafe fn from_raw_fd(fd: RawFd) -> Result<Self> {
+        Self::from_socket(Socket::from_raw_fd(fd)?)
+    }
 }
 
 impl AsRef<Socket<Stream>> for Stream {
@@ -670,6 +710,20 @@ impl AsRawFd for Stream {
     }
 }
 
+impl FromRawFd for Stream {
+    /// Constructs a new instance of `Self` from the given raw file
+    /// descriptor.
+    ///
+    /// The file descriptor must have been set to non-blocking mode.
+    ///
+    /// # Panics
+    /// Panics when the conversion fails.
+    /// Use [Stream::from_raw_fd] for a non-panicking variant.
+    unsafe fn from_raw_fd(fd: RawFd) -> Self {
+        Self::from_raw_fd(fd).expect("from_raw_fd failed")
+    }
+}
+
 impl AsyncRead for Stream {
     fn poll_read(self: Pin<&mut Self>, cx: &mut Context, buf: &mut ReadBuf) -> Poll<Result<()>> {
         self.socket.poll_recv_priv(cx, buf)
@@ -721,6 +775,19 @@ impl SeqPacketListener {
         let (socket, sa) = ready!(self.socket.poll_accept_priv(cx))?;
         Poll::Ready(Ok((SeqPacket { socket }, sa)))
     }
+
+    /// Constructs a new [SeqPacketListener] from the given raw file descriptor.
+    ///
+    /// The file descriptor must have been set to non-blocking mode.
+    ///
+    /// This function *consumes ownership* of the specified file descriptor.
+    /// The returned object will take responsibility for closing it when the object goes out of scope.
+    ///
+    /// # Safety
+    /// If the passed file descriptor is invalid, undefined behavior may occur.
+    pub unsafe fn from_raw_fd(fd: RawFd) -> Result<Self> {
+        Ok(Self { socket: Socket::from_raw_fd(fd)? })
+    }
 }
 
 impl AsRef<Socket<SeqPacket>> for SeqPacketListener {
@@ -735,6 +802,20 @@ impl AsRawFd for SeqPacketListener {
     }
 }
 
+impl FromRawFd for SeqPacketListener {
+    /// Constructs a new instance of `Self` from the given raw file
+    /// descriptor.
+    ///
+    /// The file descriptor must have been set to non-blocking mode.
+    ///
+    /// # Panics
+    /// Panics when the conversion fails.
+    /// Use [SeqPacketListener::from_raw_fd] for a non-panicking variant.
+    unsafe fn from_raw_fd(fd: RawFd) -> Self {
+        Self::from_raw_fd(fd).expect("from_raw_fd failed")
+    }
+}
+
 /// An L2CAP sequential packet socket (sequenced, reliable, two-way connection-based data transmission path for
 /// datagrams of fixed maximum length).
 #[derive(Debug)]
@@ -801,6 +882,19 @@ impl SeqPacket {
     pub fn recv_mtu(&self) -> Result<usize> {
         self.socket.recv_mtu().map(|v| v.into())
     }
+
+    /// Constructs a new [SeqPacket] from the given raw file descriptor.
+    ///
+    /// The file descriptor must have been set to non-blocking mode.
+    ///
+    /// This function *consumes ownership* of the specified file descriptor.
+    /// The returned object will take responsibility for closing it when the object goes out of scope.
+    ///
+    /// # Safety
+    /// If the passed file descriptor is invalid, undefined behavior may occur.
+    pub unsafe fn from_raw_fd(fd: RawFd) -> Result<Self> {
+        Ok(Self { socket: Socket::from_raw_fd(fd)? })
+    }
 }
 
 impl AsRef<Socket<SeqPacket>> for SeqPacket {
@@ -815,6 +909,20 @@ impl AsRawFd for SeqPacket {
     }
 }
 
+impl FromRawFd for SeqPacket {
+    /// Constructs a new instance of `Self` from the given raw file
+    /// descriptor.
+    ///
+    /// The file descriptor must have been set to non-blocking mode.
+    ///
+    /// # Panics
+    /// Panics when the conversion fails.
+    /// Use [SeqPacket::from_raw_fd] for a non-panicking variant.
+    unsafe fn from_raw_fd(fd: RawFd) -> Self {
+        Self::from_raw_fd(fd).expect("from_raw_fd failed")
+    }
+}
+
 /// An L2CAP datagram socket (connection-less, unreliable messages of a fixed maximum length).
 #[derive(Debug)]
 pub struct Datagram {
@@ -916,6 +1024,19 @@ impl Datagram {
     pub fn recv_mtu(&self) -> Result<usize> {
         self.socket.recv_mtu().map(|v| v.into())
     }
+
+    /// Constructs a new [Datagram] from the given raw file descriptor.
+    ///
+    /// The file descriptor must have been set to non-blocking mode.
+    ///
+    /// This function *consumes ownership* of the specified file descriptor.
+    /// The returned object will take responsibility for closing it when the object goes out of scope.
+    ///
+    /// # Safety
+    /// If the passed file descriptor is invalid, undefined behavior may occur.
+    pub unsafe fn from_raw_fd(fd: RawFd) -> Result<Self> {
+        Ok(Self { socket: Socket::from_raw_fd(fd)? })
+    }
 }
 
 impl AsRef<Socket<Datagram>> for Datagram {
@@ -929,3 +1050,17 @@ impl AsRawFd for Datagram {
         self.socket.as_raw_fd()
     }
 }
+
+impl FromRawFd for Datagram {
+    /// Constructs a new instance of `Self` from the given raw file
+    /// descriptor.
+    ///
+    /// The file descriptor must have been set to non-blocking mode.
+    ///
+    /// # Panics
+    /// Panics when the conversion fails.
+    /// Use [Datagram::from_raw_fd] for a non-panicking variant.
+    unsafe fn from_raw_fd(fd: RawFd) -> Self {
+        Self::from_raw_fd(fd).expect("from_raw_fd failed")
+    }
+}
